﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Data.SqlClient;

namespace SkladovaEvidence
{
    /// <summary>
    /// Přihlášení uživatele.
    /// Číslování chyb: 4000 - 4999.
    /// </summary>
    public partial class Login : Form
    {
        private Util util;
        private int iPocOpak = 0;
        private ClsUzivatel clsUziv;
        private string sIni;

        internal Login(Util pUtil)
        {
            int iErrKod = 4100;
            InitializeComponent();
            this.util = pUtil;
            this.clsUziv = new ClsUzivatel();

            this.sIni = Environment.GetEnvironmentVariable("appdata") +
              "\\SkladovaEvidence";

            iErrKod = 4101;
            if (!Directory.Exists(this.sIni))
                Directory.CreateDirectory(this.sIni);

            this.sIni += "\\Login.ini";

            try
            {
                iErrKod = 4102;
                if (File.Exists(this.sIni))
                {
                    iErrKod = 4103;
                    this.ds.Clear();
                    this.ds.ReadXml(this.sIni);

                    iErrKod = 4104;
                    if (this.ds.Tables.Contains("tabLogin") &&
                      this.ds.Tables["tablogin"].Rows.Count > 0)
                        this.tbLogin.Text = (string)this.ds.Tables["tablogin"].
                          Rows[0]["login"];
                }
            }
            catch (Exception exc)
            {
                this.util.PisLog("Login " + iErrKod.ToString() +
                  ": Chyba při načítání přihlašovacího jména: " + exc.ToString(),
                  true);
            }
        }
        private void btnZobrazit_MouseDown(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = false;
        }

        private void btnZobrazit_MouseUp(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = true;
        }

        private void btnStorno_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            this.Overeni();
        }

        private void Overeni()
        {
            int iErrKod = 4000;
            SqlCommand sqlCommand = null;
            SqlDataReader sqlDataReader = null;
            string sErr;
            DataRow row;
            Nastaveni nast = null;

            try
            {
                while (true)
                {
                    if (!this.util.PripojDb(out sqlCommand, out sErr))
                    {
                        this.util.PisLog("Login " + iErrKod.ToString() + ": " + sErr, true);

                        if (nast != null)
                            nast.Dispose();

                        nast = new Nastaveni(this.util);

                        if (nast.ShowDialog() != DialogResult.OK)
                        {
                            this.DialogResult = DialogResult.Abort;
                            return;
                        }

                        this.util.SqlConStr = nast.SqlConStr;
                    }
                    else
                    {
                        break;
                    }
                }

                iErrKod = 4001;
                sqlCommand.CommandText = "select * from tuzivatele " +
                  "where loginupper = @login";
                sqlCommand.Parameters.Clear();
                sqlCommand.Parameters.Add("@login", SqlDbType.VarChar, 30).Value =
                  this.tbLogin.Text.ToUpper().Trim();

                iErrKod = 4002;
                sqlDataReader = sqlCommand.ExecuteReader();

                iErrKod = 4003;
                if (!sqlDataReader.Read() || this.util.Desifr(sqlDataReader.GetString(sqlDataReader.GetOrdinal(
                  "heslo"))) != this.tbHeslo.Text)
                {
                    iErrKod = 4004;
                    this.util.PisLog("Chybné přihlašovací jméno nebo heslo.", true);
                    this.tbHeslo.Focus();

                    if ((++this.iPocOpak) >= 3)
                        this.DialogResult = DialogResult.Abort;

                    return;
                }

                iErrKod = 4005;
                this.clsUziv.bAdmin = sqlDataReader.GetString(sqlDataReader.GetOrdinal(
                  "b_admin")) == "A";
                this.clsUziv.bStanice = sqlDataReader.GetString(sqlDataReader.GetOrdinal(
                  "b_stanice")) == "A";
                this.clsUziv.iId = sqlDataReader.GetInt32(sqlDataReader.GetOrdinal("id"));
                this.clsUziv.sLogin = sqlDataReader.GetString(sqlDataReader.GetOrdinal("login"));
                this.clsUziv.sJmeno = sqlDataReader.GetString(sqlDataReader.GetOrdinal("jmeno"));
                this.clsUziv.sPrijmeni = sqlDataReader.GetString(sqlDataReader.GetOrdinal("prijmeni"));
                this.clsUziv.sTitulPredJmenem = sqlDataReader.GetString(sqlDataReader.GetOrdinal(
                  "titulpredjm"));
                this.clsUziv.sTitulZaJmenem = sqlDataReader.GetString(sqlDataReader.GetOrdinal(
                  "titulzajm"));
                this.clsUziv.sHeslo = this.util.Desifr(sqlDataReader.GetString(sqlDataReader.GetOrdinal(
                  "heslo")));

                iErrKod = 4006;
                this.ds.Tables["tablogin"].Clear();
                row = this.ds.Tables["tablogin"].NewRow();
                row["login"] = this.tbLogin.Text;
                this.ds.Tables["tablogin"].Rows.Add(row);
                iErrKod = 4007;
                this.ds.WriteXml(this.sIni);

                // Pokud se při přihlašování opravovaly parametry a pro připojení 
                // k databázi a přihlásil-li se administrátor, uložím je.
                if (nast != null && this.clsUziv.bAdmin)
                    nast.UlozDtb();

                this.DialogResult = DialogResult.OK;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Login: Při přihlašování vznikla chyba: " +
                  iErrKod.ToString() + " - " + exc.ToString(), true);
                this.DialogResult = DialogResult.Abort;
            }
            finally
            {
                this.util.OdpojDb(sqlCommand, sqlDataReader);

                if (nast != null)
                    nast.Dispose();
            }
        }

        internal ClsUzivatel ClsUziv
        {
            get
            {
                return this.clsUziv;
            }
        }

        private void Login_Shown(object sender, EventArgs e)
        {
            if (this.tbLogin.Text == "")
                this.tbLogin.Focus();
            else
                this.tbHeslo.Focus();
        }

        private void tbLogin_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.tbHeslo.Focus();
        }

        private void tbHeslo_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.Overeni();
        }

        private void Login_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                this.DialogResult = DialogResult.Cancel;
        }
    }
}
