﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SkladovaEvidence
{
    /// <summary>
    /// Hlavní formulář
    /// Číslování chyb: 1000 - 1999
    /// </summary>
    internal partial class Main : Form
    {
        private List<Form> LForm = new List<Form>();
        private Util util;

        public Main()
        {
            int iErrKod = 1000;
            InitializeComponent();
            FileVersionInfo verze;
            try
            {
                this.util = new Util();
                iErrKod = 10001;
                verze = FileVersionInfo.GetVersionInfo(Application.ExecutablePath);
                this.Text = "Sklad evidence v. " + verze.FileMajorPart.ToString() +
                    "." + verze.FileMinorPart.ToString() + "." + verze.FileBuildPart.ToString() +
                    " - " + this.util.KdoZmenil;
            }
            catch (Exception exc)
            {
                MessageBox.Show("Chyba při spouštění programu \"SkladovaEvidence\":" + iErrKod.ToString() + " - " + exc.ToString());
                Environment.Exit(iErrKod);
            }
        }
        /// <summary>
        /// Zobrazí formulář se zadaným jménem, Pokud formulář neni v Form, nejdříve ho vytvoří.
        /// </summary>
        /// <param name="pName">Název formuláře</param>
        /// <returns></returns>
        private Form OtevriForm(string pName)
        {
            int iErrKod = 1100;
            int i1;
            Form f;

            for (i1 = 0; i1 < this.LForm.Count && this.LForm[i1].Name != pName; ++i1) ;

            if (i1 >= this.LForm.Count)
            {
                switch (pName)
                {
                    case "Prijem":
                        iErrKod = 10101;
                        Prijem prijem = new Prijem(this.util);
                        f = prijem;
                        break;
                    case "Vydej":
                        iErrKod = 10102;
                        Vydej vydej = new Vydej(this.util);
                        f = vydej;
                        break;
                    case "Zbozi":
                        iErrKod = 10103;
                        Zbozi zbozi = new Zbozi(this.util);
                        f = zbozi;
                        break;
                    case "FyzOsoby":
                        iErrKod = 10104;
                        Zakaznici zakazniciFyz = new Zakaznici(this.util, EnZakaznici.fyzOsoby);
                        zakazniciFyz.Name = "Fyzické osoby";
                        f = zakazniciFyz;
                        break;
                    case "PravOsoby":
                        iErrKod = 10104;
                        Zakaznici zakazniciPrav = new Zakaznici(this.util, EnZakaznici.pravOsoby);
                        zakazniciPrav.Name = "Právnické osoby";
                        f = zakazniciPrav;
                        break;
                    case "Dodavatele":
                        iErrKod = 10105;
                        Dodavatele dodavatele = new Dodavatele(this.util);
                        f = dodavatele;
                        break;
                    case "Uzivatele":
                        iErrKod = 10106;
                        Uzivatele uzivatele = new Uzivatele(this.util);
                        f = uzivatele;
                        break;
                    case "Nastaveni":
                        iErrKod = 10107;
                        Nastaveni nastaveni = new Nastaveni(this.util);
                        f = nastaveni;
                        break;
                    default:
                        iErrKod = 10109;
                        return null;
                }
                this.LForm.Add(f);
            }
            this.LForm[i1].Show();
            this.LForm[i1].BringToFront();
            return this.LForm[i1];
        }

        private void mnuPrijem_Click(object sender, EventArgs e)
        {
            this.OtevriForm("Prijem");
        }

        private void mnuVydej_Click(object sender, EventArgs e)
        {
            this.OtevriForm("Vydej");
        }

        private void mnuZbozi_Click(object sender, EventArgs e)
        {
            this.OtevriForm("Zbozi");
        }

        private void mnuFyzOsoba_Click(object sender, EventArgs e)
        {
            this.OtevriForm("FyzOsoby");
        }

        private void mnuPravOsoba_Click(object sender, EventArgs e)
        {
            this.OtevriForm("PravOsoby");
        }

        private void mnuDodavatele_Click(object sender, EventArgs e)
        {
            this.OtevriForm("Dodavatele");
        }

        private void mnuUzivatele_Click(object sender, EventArgs e)
        {
            this.OtevriForm("Uzivatele");
        }

        private void mnuNastaveni_Click(object sender, EventArgs e)
        {
            this.OtevriForm("Nastaveni");
        }

        private void mnuZmenaHesla_Click(object sender, EventArgs e)
        {
            ZmenaHesla zmenaHesla = new ZmenaHesla(this.util);
            try
            {
              zmenaHesla.ShowDialog();
            }
            finally
            {
              zmenaHesla.Dispose();
            }
        }

        private void Main_FormClosing(object sender, FormClosingEventArgs e)
        {
            StringBuilder sb = new StringBuilder();
            foreach (Form f in this.LForm)
            {
                if (!((IForm)f).LzeUkoncit)
                {
                   f.Show();
                   f.BringToFront();
                   e.Cancel = true;
                   sb.AppendLine(f.Text);
                }
               else
                {
                   ((IForm)f).Konec = true;
                    f.Close();
                }
            }
        }
    }
}
