﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using System.Data;
using System.Net;
using System.Net.Sockets;
using System.Data.SqlClient;

namespace SkladovaEvidence
{
    internal enum EnZakaznici : byte { pravOsoby, fyzOsoby };

    /// <summary>
    /// Společné a pomocné metody.
    /// Číslování chyb: 2000 - 2999.
    /// </summary>
    internal class Util
    {
        // Kritická sekce pro zápis do logu.
        private object CSLog = new object();
        private string sLog = "", sLogKopie = "", sLogDir = "";
        private ClsUzivatel clsUzivatel;
        private string sSqlConStr = "";

        internal Util()
        {
            int iErrKod = 2100;
            Nastaveni nastaveni = null;
            Login login = null;

            try
            {
                this.clsUzivatel = new ClsUzivatel();

                // Když se nepodaří načíst parametry pro připojení k databázi, musí je uživatel zadat. Teprve potom je možné požádat o login.
                if (!this.NactiDtb())
                {
                    iErrKod = 20101;
                    nastaveni = new Nastaveni(this);

                    if (nastaveni.ShowDialog() != DialogResult.OK)
                    {
                        iErrKod = 20102;
                        Environment.Exit(iErrKod);
                    }

                    iErrKod = 20103;
                    SqlConStr = nastaveni.SqlConStr;
                    login = new Login(this);

                    iErrKod = 20104;
                    if (login.ShowDialog() != DialogResult.OK)
                       Environment.Exit(iErrKod);

                    iErrKod = 20105;
                    if (login.ShowDialog() != DialogResult.OK)
                       Environment.Exit(iErrKod);

                    this.clsUzivatel = login.ClsUziv;

                    // Když se přihlásil admin, parametry pro připojení k databázi uložím.
                    if (this.clsUzivatel.bAdmin)
                        nastaveni.UlozDtb();
                }
                else
                {
                    login = new Login(this);

                    iErrKod = 20106;
                    if (login.ShowDialog() != DialogResult.OK)
                        Environment.Exit(iErrKod);

                    this.clsUzivatel = login.ClsUziv;
                }
            }
            catch (Exception exc)
            {
                throw new Exception("Util" + iErrKod.ToString() + ": " + exc.ToString());
            }
            finally
            {
                if (nastaveni != null)
                    nastaveni.Dispose();

                if (login != null)
                    login.Dispose();
            }
        }

        /// <summary>
        /// Vrátí data přihlášeného uživatele. Pokud ještě uživatel není přihlášený,
        /// vrátí inicializovanou instanci.
        /// </summary>
        internal ClsUzivatel ClsUziv
        {
           get
           {
              return this.clsUzivatel;
           }
        }

        /// <summary>
        /// Vrátí připojovací řetězec (connection string) do databáze
        /// </summary>
        internal string SqlConStr
        {
            get
            {
                return this.sSqlConStr;
            }
            set
            {
                this.sSqlConStr = value;
            }
        }

        /// <summary>
        /// Vrátí poskládaný text KdoZmenil pro zápis do databáze.
        /// </summary>
        internal string KdoZmenil
        {
            get
            {
                return (this.ClsUziv.sLogin + " (" +
                  (this.ClsUziv.sTitulPredJmenem != "" ?
                  this.ClsUziv.sTitulPredJmenem + " " : "") +
                  (this.ClsUziv.sPrijmeni != "" ?
                  this.ClsUziv.sPrijmeni + " " : "") +
                  (this.ClsUziv.sJmeno != "" ?
                  this.ClsUziv.sJmeno + " " : "") +
                  (this.ClsUziv.sTitulZaJmenem != "" ?
                  this.ClsUziv.sTitulZaJmenem : "")).Trim() + ")";
             }
         }

        /// <summary>
        /// Vrátí DateTime KdyZmenil pro zápis do databáze.
        /// </summary>
        internal DateTime KdyZmenil
        {
            get
            {
                DateTime aktualniDateTime = DateTime.Now;
                return aktualniDateTime;
            }
        }

        /// <summary>
        /// Vrátí SqlCommand s naplněnými hodnotami Connection a Transaction a otevřenou databází.
        /// </summary>
        /// <param name="sqlComman">Vytvořený FbCommand</param>
        /// <param name="pErr">Chybová zpráva</param>
        /// <returns></returns>
        internal bool PripojDb(out SqlCommand sqlCommand, out string pErr)
        {
            pErr = "";
            sqlCommand = null;

            try
            {
                sqlCommand = new SqlCommand();
                sqlCommand.Connection = new SqlConnection(this.SqlConStr);
                sqlCommand.Connection.Open();
                sqlCommand.Transaction = sqlCommand.Connection.BeginTransaction();
                return true;
            }
            catch (Exception exc)
            {
                pErr = "PripojDb: Při připojování k databázi došlo k chybě: " + exc.ToString();
                return false;
            }
        }

        /// <summary>
        /// Zavře FbDataReader, pokud je zadaný, udělá Rollback na transakci a zavře připojení k databázi.
        /// </summary>
        /// <param name="sqlCommand">FbCommand, kterým se pracovalo s databází</param>
        /// <param name="sqlDataReader">Instance FbDataReaderu, nebo null</param>
        internal void OdpojDb(SqlCommand sqlCommand, SqlDataReader sqlDataReader = null)
        {
            if (sqlDataReader != null)
                sqlDataReader.Close();

            if (sqlCommand?.Transaction?.Connection != null)
                sqlCommand.Transaction.Rollback();

            if (sqlCommand?.Connection != null)
                sqlCommand.Connection.Dispose();
        }

        /// <summary>
        /// Načte parametry pro připojení k databázi
        /// </summary>
        /// <returns></returns>
        private bool NactiDtb()
        {
            int iErrKod = 2200;
            DataSet ds = new DataSet();
            //Parametry jsou uložené v xml souboru SkladovaEvidence.dtb, který je ve stejném adresáři jako spouštěný program.
            string sSoub = Application.StartupPath + "\\SkladovaEvidence.dtb";
            SqlConnectionStringBuilder csb;

            try
            {
                if (!File.Exists(sSoub))
                {
                    iErrKod = 2201;
                    this.PisLog("NactiDtb " + iErrKod.ToString() + ": Nebyl nalezen " +
                    "soubor \"" + sSoub + "\" s parametry pro připojení k databázi. " +
                    "Pro spuštění programu je nutné parametry zadat.", true);
                    return false;
                }
                iErrKod = 2202;
                ds.ReadXml(sSoub);

                iErrKod = 2203;
                if (!ds.Tables.Contains("dtb") || ds.Tables["dtb"].Rows.Count != 1)
                {
                    iErrKod = 2004;
                    this.PisLog("NactiDtb " + iErrKod.ToString() + ": Soubor \"" + sSoub +
                    "\" s parametry pro připojení k databázi neobsahuje tabulku " +
                    "\"Dtb\" nebo tabulka neobsahuje právě jeden řádek. " +
                    "Pro spuštění programu je nutné parametry zadat.", true);
                    return false;
                }

                csb = new SqlConnectionStringBuilder();
                csb.DataSource = (string)ds.Tables["dtb"].Rows[0]["server"];
                csb.InitialCatalog = (string)ds.Tables["dtb"].Rows[0]["database"];
                csb.IntegratedSecurity = true;
                SqlConStr = csb.ConnectionString;
                return true;
            }
            catch (Exception exc)
            {
                this.PisLog("NactiDtb " + iErrKod.ToString() + ": Při načítání " +
                "parametrů pro připojení k databázi ze souboru \"" + sSoub +
                "\" došlo k chybě. Pro spuštění programu je nutné parametry zadat." +
                Environment.NewLine + exc.ToString(), true);
                return false;
            }
            finally
            {
                ds.Dispose();
            }
        }

        /// <summary>
        /// Zašifruje zadaný text.
        /// </summary>
        /// <param name="pText">Šifrovaný text.</param>
        /// <returns>Zašifrovaný text.</returns>
        internal string Sifr(string pText)
        {
            char[] cMaska = new char[2];
            int i1, i2;
            char cLen;
            StringBuilder sb;
            Random rand = new Random();

            sb = new StringBuilder();
            cMaska[0] = cMaska[1] = (char)0;

            cMaska[0] = (char)rand.Next(1, char.MaxValue);
            cMaska[1] = (char)rand.Next(1, char.MaxValue);

            sb.Append(this.CharToHexStr(cMaska[0]) + this.CharToHexStr(cMaska[1]));

            // Zapíšu zašifrovanou délku textu.
            cLen = (char)pText.Length;
            sb.Append(this.CharToHexStr((char)(cLen ^ cMaska[0])));
            // Pozice, na kterou se budou vkládat náhodné znaky pro doplnění délky.
            i2 = sb.Length;

            for (i1 = 0; i1 < pText.Length; ++i1)
                sb.Append(this.CharToHexStr((char)(pText[i1] ^ cMaska[i1 % 2])));

            // Doplním délku zašifrovaného textu na 48 znaků a spočítám počet 
            // vložených znaků.
            cLen = (char)0;

            while (sb.Length < 48)
            {
                sb.Insert(i2, this.CharToHexStr((char)rand.Next(1, char.MaxValue)));
                ++cLen;
            }

            sb.Insert(i2, this.CharToHexStr((char)(cLen ^ cMaska[1])));

            return sb.ToString();
        }

        /// <summary>
        /// Rozšifruje zadaný text.
        /// </summary>
        /// <param name="pText">Zašifrovaný text.</param>
        /// <returns>Rozšifrovaný text.</returns>
        internal string Desifr(string pText)
        {
            char[] cMaska = new char[2];
            List<char> lc = new List<char>();
            int i1, i2;
            StringBuilder sb;
            ushort uLen = ushort.MaxValue, uPosun = (ushort)0;
            string sRet = "";

            if (pText.Length < 12 || pText.Length % 4 != 0)
                return sRet;

            sb = new StringBuilder();

            try
            {
                // K délce zašifrovaného textu přičtu 2 znaky na masku a 1 znak na délku
                // textu a 1 znak na počet vložených znaků.
                for (i1 = 0; i1 < pText.Length / 4 && i1 < uLen + 4; ++i1)
                {
                    lc.Clear();

                    for (i2 = (i1 + uPosun) * 4; i2 < (i1 + uPosun) * 4 + 4; ++i2)
                        lc.Add(pText[i2]);

                    switch (i1)
                    {
                        case 0:
                            cMaska[0] = this.StrHexToChar(lc.ToArray());
                            break;

                        case 1:
                            cMaska[1] = this.StrHexToChar(lc.ToArray());
                            break;

                        case 2: // Délka textu.
                            uLen = (ushort)(this.StrHexToChar(lc.ToArray()) ^ cMaska[0]);
                            break;

                        case 3: // Počet vložených náhodných znaků.
                            uPosun = (ushort)(this.StrHexToChar(lc.ToArray()) ^ cMaska[1]);
                            break;

                        default:
                            sb.Append((char)(this.StrHexToChar(lc.ToArray()) ^ cMaska[i1 % 2]));
                            break;
                    }
                }

                sRet = sb.ToString();

                if (sRet.Length != uLen)
                    sRet = "";
            }
            catch
            {
                sRet = "";
            }

            return sRet;
        }

        /// <summary>
        /// Převede textové vyjádření hex-tvaru na znak.
        /// </summary>
        /// <param name="pStr">4 hex znaky.</param>
        /// <returns>Převedený znak.</returns>
        private char StrHexToChar(char[] pStr)
        {
            int i1, iLen;
            char cVysl = (char)0;

            iLen = Math.Min(4, pStr.Length);

            for (i1 = 0; i1 < iLen; ++i1)
                cVysl += (char)(this.HexToByte(pStr[i1]) << (iLen - i1 - 1) * 4);

            return cVysl;
        }

        /// <summary>
        /// Převede hex znak na byte.
        /// </summary>
        /// <param name="pZnak">Znak.</param>
        /// <returns>Byte.</returns>
        private byte HexToByte(char pZnak)
        {
            char c = char.ToUpper(pZnak);

            if (c >= '0' && c <= '9')
            {
                return (byte)(c - '0');
            }
            else
            {
                if (c >= 'A' && c <= 'F')
                    return (byte)(c - 'A' + 10);
                else
                    return (byte)0;
            }
        }

        /// <summary>
        /// Převede znak na vyjádření v hex-tvaru.
        /// </summary>
        /// <param name="pZnak">Převáděný znak.</param>
        /// <returns>Hex-tvar znaku.</returns>
        private string CharToHexStr(char pZnak)
        {
            StringBuilder sb = new StringBuilder();
            byte b1;

            b1 = (byte)(pZnak >> 8);
            sb.Append(b1.ToString("X2"));

            b1 = (byte)(pZnak & 0x00FF);
            sb.Append(b1.ToString("X2"));

            return sb.ToString();
        }

        internal DialogResult PisLog(string pText, bool pZprava, MessageBoxButtons pMsgButton = MessageBoxButtons.OK,
            MessageBoxIcon pMsgIcon = MessageBoxIcon.Error)
        {
            int iErrKod = 2300;
            StreamWriter sw = null;
            FileInfo fi = null;

            SqlCommand sqlCommand = null;
            string sErr, sIPAdr = "", sTyp;
            int i1;
            IPHostEntry ipHE;
            DialogResult dlgRes = DialogResult.None;

            // Má-li se zobrazit MessageBox, udělám to.
            if (pZprava)
            {
                dlgRes = MessageBox.Show(pText, "SkladovaEvidence", pMsgButton, pMsgIcon);
                pText += Environment.NewLine + " Odpověď: " + dlgRes.ToString();
            }
            for (i1 = 0; i1 < 10; ++i1)
            {
                try
                {
                    try
                    {
                        Monitor.Enter(CSLog);
                        iErrKod = 2301;
                        // Nastaví úplnou cestu k logovému a archivnímu souboru do adresáře
                        // v AppData, pokud ještě nejsou cesty nastavené.
                        if (this.sLog == "" || this.sLogKopie == "" || this.sLogDir == "")
                        {
                            iErrKod = 2302;
                            this.sLogDir = Environment.GetEnvironmentVariable("appdata") +
                              "\\SkladovaEvidence";

                            iErrKod = 20303;
                            if (!Directory.Exists(this.sLogDir))
                                Directory.CreateDirectory(this.sLogDir);

                            iErrKod = 2304;
                            this.sLog = this.sLogDir + "\\SkladovaEvidence.log";
                            this.sLogKopie = this.sLogDir + "\\SkladovaEvidence.zal";
                        }

                        // Pokud je log příliš dlouhý, přejmenuju ho.
                        iErrKod = 2305;
                        if (File.Exists(this.sLog))
                        {
                            fi = new FileInfo(this.sLog);

                            iErrKod = 2306;
                            if (fi.Length > 1000000)
                            {
                                iErrKod = 2307;
                                // Zruším starý záložní.
                                File.Delete(this.sLogKopie);
                                iErrKod = 20308;
                                // Z aktuálního udělám záložní.
                                File.Move(this.sLog, this.sLogKopie);
                            }
                        }
                        try
                        {
                            iErrKod = 2309;
                            sw = new StreamWriter(this.sLog, true, Encoding.GetEncoding(1250));
                            iErrKod = 2310;
                            sw.WriteLine(DateTime.Now.ToString("dd.MM.yyyy HH:mm:ss.fff") +
                              ": " + pText);
                            break;
                        }
                        finally
                        {
                            if (sw != null)
                            {
                                sw.Close();
                                sw.Dispose();
                            }
                        }
                    }
                    finally
                    {
                        Monitor.Exit(CSLog);
                    }
                }
                catch (Exception exc)
                {
                    if (pZprava && i1 == 9)
                    {
                        return MessageBox.Show("PisLog" + iErrKod.ToString() + ": " +
                          exc.ToString(), "PisLog", MessageBoxButtons.OK,
                          MessageBoxIcon.Error);
                    }
                    else
                    {
                        Thread.Sleep(10);
                    }
                }
            }

            // Zápis do tabulky TLOG, aby měl administrátor přehled o chybách.
            try
            {
                // Nebudu řešit, když se nepodaří otevřít databázi nebo zápis skončí 
                // s chybou.
                if (this.PripojDb(out sqlCommand, out sErr))
                {
                    // Zjistím lokální IP adresu.
                    ipHE = Dns.GetHostEntry(Dns.GetHostName());

                    // Přednostně budu hledat IPv4.
                    foreach (IPAddress ip in ipHE.AddressList)
                    {
                        if (ip.AddressFamily == AddressFamily.InterNetwork)
                        {
                            sIPAdr = ip.ToString();
                            break;
                        }
                    }

                    // Když se nenašla IPv4, budu hledat IPv6.
                    if (sIPAdr == "")
                    {
                        foreach (IPAddress ip in ipHE.AddressList)
                        {
                            if (ip.AddressFamily == AddressFamily.InterNetworkV6)
                            {
                                sIPAdr = ip.ToString();
                                break;
                            }
                        }
                    }
                    // Typ zprávy převezmu z ikony.
                    switch (pMsgIcon)
                    {
                        case MessageBoxIcon.Information:
                            sTyp = "I";
                            break;

                        case MessageBoxIcon.Error:
                            sTyp = "E";
                            break;

                        case MessageBoxIcon.Question:
                            sTyp = "Q";
                            break;

                        case MessageBoxIcon.Warning:
                            sTyp = "W";
                            break;

                        default:
                            sTyp = "O";
                            break;
                    }

                    sqlCommand.CommandText = "insert into tlog (IPADRESA, UZIVATEL, " +
                      "POCITAC, TYP, TEXT) " +
                      "values (@IPADRESA, @UZIVATEL, @POCITAC, @TYP, @TEXT)";
                    sqlCommand.Parameters.Add("@ipadresa", SqlDbType.VarChar, 50).Value =
                      sIPAdr;
                    sqlCommand.Parameters.Add("@uzivatel", SqlDbType.VarChar, 200).Value =
                        this.KdoZmenil;
                    sqlCommand.Parameters.Add("@pocitac", SqlDbType.VarChar, 100).Value =
                      Environment.MachineName;
                    sqlCommand.Parameters.Add("@typ", SqlDbType.Char, 1).Value = sTyp;
                    sqlCommand.Parameters.Add("@text", SqlDbType.VarChar, 32000).Value = pText;
                    sqlCommand.ExecuteNonQuery();
                    sqlCommand.Transaction.Commit();
                }
            }
            catch
            {
            }
            finally
            {
                this.OdpojDb(sqlCommand);
            }
            return dlgRes;
        }

        /// <summary>
        /// Načte z tabulky TNASTAVENI hodnotu se zadaným názvem pro zadaného
        /// uživatele.
        /// </summary>
        /// <param name="pIdUziv">Id uživatele (-1 = globální hodnota).</param>
        /// <param name="pNazev">Název hodnoty.</param>
        /// <param name="pHodnota">Načtená hodnota.</param>
        /// <param name="pErr">Chybová zpráva.</param>
        /// <returns></returns>
        internal bool NactiHodnotu(int pIdUziv, string pNazev, out string pHodnota,
          out string pErr)
        {
            SqlCommand sqlCommand = null;
            SqlDataReader sqlDataReader = null;
            string sErr;
            pHodnota = pErr = "";

            try
            {
                if (!this.PripojDb(out sqlCommand, out sErr))
                {
                   pErr = "NactiHodnotu: " + sErr;
                   return false;
                }

                sqlCommand.CommandText = "select hodnota from tnastaveni " +
                  "where iduziv = @iduziv and nazev = @nazev";
                sqlCommand.Parameters.Add("@iduziv", SqlDbType.Int).Value = pIdUziv;
                sqlCommand.Parameters.Add("@nazev", SqlDbType.VarChar, 100).Value =
                  pNazev.ToUpper();
                sqlDataReader = sqlCommand.ExecuteReader();

                if (sqlDataReader.Read())
                    pHodnota = sqlDataReader.GetString(sqlDataReader.GetOrdinal("hodnota"));

                return true;
            }
            catch (Exception exc)
            {
                pErr = "NactiHodnotu: " + exc.ToString();
                return false;
            }
            finally
            {
                this.OdpojDb(sqlCommand, sqlDataReader);
            }
        }

        /// <summary>
        /// Uloží do tabulky TNASTAVENI hodnotu se zadaným názvem pro zadaného
        /// uživatele.
        /// </summary>
        /// <param name="pIdUziv">Id uživatele (-1 = globální hodnota).</param>
        /// <param name="pNazev">Název hodnoty.</param>
        /// <param name="pHodnota">Zapisovaná hodnota.</param>
        /// <param name="pErr">Chybová zpráva.</param>
        /// <returns></returns>
        internal bool ZapisHodnotu(int pIdUziv, string pNazev, string pHodnota,
          out string pErr)
        {
            SqlCommand sqlCommand = null;
            string sErr;
            pErr = "";

            try
            {
                if (!this.PripojDb(out sqlCommand, out sErr))
                {
                    pErr = "ZapisHodnotu: " + sErr;
                    return false;
                }

                sqlCommand.CommandText = "update or insert into tnastaveni " +
                  "(iduziv, nazev, hodnota, kdozmenil) " +
                  "values (@iduziv, @nazev, @hodnota, @kdozmenil) " +
                  "matching (iduziv, nazev)";
                sqlCommand.Parameters.Add("@iduziv", SqlDbType.Int).Value = pIdUziv;
                sqlCommand.Parameters.Add("@nazev", SqlDbType.VarChar, 100).Value =
                  pNazev.ToUpper();
                sqlCommand.Parameters.Add("@hodnota", SqlDbType.VarChar, 8000).Value =
                  pHodnota;
                sqlCommand.Parameters.Add("@kdozmenil", SqlDbType.VarChar, 200).Value =
                  this.KdoZmenil;
                sqlCommand.ExecuteNonQuery();
                sqlCommand.Transaction.Commit();
                return true;
            }
            catch (Exception exc)
            {
                pErr = "ZapisHodnotu: " + exc.ToString();
                return false;
            }
            finally
            {
                //this.OdpojDb(fbCmd);
            }
        }

        /// <summary>
        /// Rozloží řádek CSV na jednotlivá pole.
        /// </summary>
        /// <param name="pRadek">Řádek CSV souboru.</param>
        /// <returns></returns>
        internal string[] RozlozCSV(string pRadek)
        {
            int i1;
            bool bUvozovky = false;
            StringBuilder sb = new StringBuilder();
            List<string> lPole = new List<string>();

            for (i1 = 0; i1 < pRadek.Length; ++i1)
            {
                // Uvozovky na začátku nebo na konci údaje.
                if (pRadek[i1] == '"' && (i1 == pRadek.Length - 1 ||
                  pRadek[i1 + 1] != '"'))
                {
                    bUvozovky = !bUvozovky;
                    continue;
                }

                // Středník jako oddělovač údajů.
                if (!bUvozovky && (pRadek[i1] == ';' || pRadek[i1] == ','))
                {
                    lPole.Add(sb.ToString());
                    sb.Clear();
                    continue;
                }

                sb.Append(pRadek[i1]);

                // Zdvojené uvozovky uprostřed textu zapíšu jen jednou.
                if (pRadek[i1] == '"' && pRadek[i1 + 1] == '"')
                    ++i1;
            }

            // Poslední rozpracovaný údaj.
            lPole.Add(sb.ToString());

            return lPole.ToArray();
        }
    }
}

