﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Data.SqlClient;

namespace SkladovaEvidence
{
    /// <summary>
    /// Správa změna hesla.
    /// Číslování chyb: 13000 - 13999.
    /// </summary>
    internal partial class ZmenaHesla : Form
    {
        private Util util;

        public ZmenaHesla(Util pUtil)
        {
            InitializeComponent();
            this.util = pUtil;
        }
        private void btnZobrazit_MouseDown(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = false;
        }

        private void btnZobrazit_MouseUp(object sender, MouseEventArgs e)
        {
            this.tbHeslo.UseSystemPasswordChar = true;
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            this.UlozData();
        }

        private bool UlozData()
        {
            int iErrKod = 13100;
            string sErr;
            SqlCommand sqlCommand = null;
            try
            {
                iErrKod = 13101;
                if (this.util.Sifr(this.tbHesloPuvodni.Text) == this.util.ClsUziv.sHeslo)
                {
                    this.util.PisLog("Chybně vyplněné původní heslo.", true);
                    this.tbHeslo.Focus();
                    return false;
                }

                if (this.tbHeslo.Text == "")
                {
                    this.util.PisLog("Heslo musí být zadané.", true);
                    this.tbHeslo.Focus();
                    return false;
                }

                if (this.tbHeslo.Text != this.tbHesloZnovu.Text)
                {
                    this.util.PisLog("Nesouhlasí heslo a opakované heslo.", true);
                    this.tbHeslo.Focus();
                    return false;
                }

                iErrKod = 13102;
                if (!this.util.PripojDb(out sqlCommand, out sErr))
                {
                    iErrKod = 13103;
                    this.util.PisLog("Uzivatele " + iErrKod.ToString() + ": " + sErr,
                      true);
                    return false;
                }

                iErrKod = 13104;
                sqlCommand.CommandText = "update tuzivatele set HESLO = @heslo where id = @id";
                sqlCommand.Parameters.Add("@heslo", SqlDbType.VarChar, 200).Value = this.util.Sifr(this.tbHeslo.Text);
                sqlCommand.Parameters.Add("@id", SqlDbType.Int).Value = Convert.ToInt32(this.util.ClsUziv.iId);

                sqlCommand.ExecuteNonQuery();
                sqlCommand.Transaction.Commit();

                this.tbHeslo.Text = this.tbHesloPuvodni.Text = this.tbHesloZnovu.Text = "";
                return true;
            }
            catch (Exception exc)
            {
                this.util.PisLog("Změna hesla " + iErrKod.ToString() + ". " +
                  exc.ToString(), true);
                return false;
            }
            finally
            {
                this.util.PisLog("Změna hesla se podařila.", true,
                      MessageBoxButtons.OK, MessageBoxIcon.Information);
                this.util.OdpojDb(sqlCommand);
            }
        }

        private void btnStorno_Click(object sender, EventArgs e)
        {
            this.tbHeslo.Text = "";
            this.tbHesloPuvodni.Text = "";
            this.tbHesloZnovu.Text = "";
        }
    }
}
